/* 
 * CellInfo.java
 *
 * Copyright 2003, Leigh L. Klotz, Jr..  All Rights Reserved.
 *   Email: klotz@graflex.org
 *     Web: http: //www.graflex.org/klotz
 *
 * {LICENSE}
 */
package com.hipme.cellinfo;

import danger.app.*;
import danger.app.Registrar;
import danger.app.IPCIncoming;
import danger.app.IPCMessage;
import danger.ui.*;
import danger.net.*;
import danger.util.*;
import danger.util.format.StringFormat;
import danger.util.DEBUG;

import java.util.Vector;

/**
 * Implements the main application class.
 *
 */
public class CellInfo extends Application implements Resources, Events {
  final static String OLD_DATASTORE_NAME = "POSTS";
  final static String RECORDS_DATASTORE_NAME = "RECORDS";
  MainWindow mainWindow;

  /**
   * Constructor
   */
  public CellInfo() {
    System.err.println( "CellInfo: starting up" ); 
    DataStore ds = getDataStore();
    if (ds == null) {
      DEBUG.p("CellInfo: creating data store");
      ds = DataStore.createDataStore(RECORDS_DATASTORE_NAME, true);
    }
    if (DataStore.findDataStore(OLD_DATASTORE_NAME) != null && 
	(DataStore.findDataStore(RECORDS_DATASTORE_NAME) == null ||
	 (DataStore.findDataStore(OLD_DATASTORE_NAME).getRecordCount() > 0 &&
	  DataStore.findDataStore(RECORDS_DATASTORE_NAME).getRecordCount() == 0))) {
      upgrade(DataStore.findDataStore(OLD_DATASTORE_NAME), ds);
    }
    Registrar.registerProvider("location" , this , WHAT_LOCATION_REQUEST);
  }
	
  /*
   * Listener API
   */
  public boolean receiveEvent(Event event) {
    boolean consumed = true;
    switch (event.type) {
    case kCmd_Help:
      mainWindow.help(); 
      break;
    case kCmdNew:
      mainWindow.showCellInfo(); 
      break;
    case kUpdateList:
      mainWindow.sendUpdate(); 
      break;
    case kCmdShowAllTowers:
      mainWindow.showAllTowers(event); 
      break;
    case kCmdShowMapDialog:
      mainWindow.showMapDialog(); 
      break;
    case kShowLockedBundleAlert:
      showLockedBundleAlert();
      break;
    case kShowDiscardEntry:
      mainWindow.showDiscardEntry();
      break;
    case Event.EVENT_MESSAGE:
      consumed = handleIPC(event);
      break;
    case kLockedBundleExit:
    case kReturnToLauncher: 
      returnToLauncher();
      break;
    default: 
      consumed = false;
    }
    System.err.println("Got event e.type"+event.type+" event="+event);
    event.debugPrint();
    return consumed || super.receiveEvent(event);
  }

  private void showLockedBundleAlert() {
    AlertWindow a = getAlert(kLockedBundleAlert, this);
    a.setMessage(StringFormat.fromRsrc(kLockedBundleAlertText, getString(AppResources.ID_APP_NAME), getString(kLockedBundleUser)));
    a.show();
  }


  /**
   * Application API
   */
  public void resume() {
    // Just create the window, if it does not already exist
    if (mainWindow == null) {
      // Create a new window and bring it to the foreground
      mainWindow = new MainWindow(this);
      mainWindow.setTitle(getString(kStrCellInfo));
      mainWindow.show();
    } else 
      mainWindow.sendUpdate();

    // check for locked bundle
    String user = getString(kLockedBundleUser);
    if (user.length()>0 && !HiptopConnection.getUserName().equals(user)) {
      new Event(this, kShowLockedBundleAlert).sendToTarget();
    }
  }

  public final void networkEvent(Object s) {
    System.err.println("Got networkEvent " + s);
    if (s instanceof HTTPTransaction) {
      HTTPTransaction t = (HTTPTransaction) s;
      if (t.getResponse() == 200) {
	mainWindow.handleResponse(t);
      }
      else {
	new AlertWindow("Error: "+t.getResponseMessage()+"\n"+
			t.getString()).show();
      }
    }
  }

  DataStore getDataStore() {
    return DataStore.findDataStore(RECORDS_DATASTORE_NAME);
  }

  void upgrade(DataStore oldData, DataStore newRecords) {
    int oldCount = oldData.getRecordCount();
    for (int i = 0; i < oldCount; i++) {
      Data data = new Data(oldData.getRecordData(i));
      String s = data.getAddEditPost();
      DEBUG.p("cellinfo: upgrading ds["+i+"]="+s);
      Record record = new Record(s);
      // recordUID, if present, as we are moving to a new DataStore.
      // 0 is apparently a reserved value for DataStore UIDs.
      record.setRecordUID(0);
      DEBUG.p("cellinfo: new record =" + record.getAddEditPost());
      int newid = newRecords.addRecord(record.getBytes());
      DEBUG.p("cellinfo: new i=" + newid);
    }
  }

  final boolean handleIPC(Event event) {
    IPCIncoming ipcIncoming = ((IPCIncoming)(event.argument));
    IPCMessage ipcMessage = ipcIncoming.getMessage();
    boolean handled = false;
    DEBUG.p("CellInfo handleIPC event.what = " + event.what);
    event.debugPrint();
    handled = true;
    String action = ipcMessage.findString("action");
    String userName = ipcMessage.findString("userName");
    String text = StringFormat.fromRsrc(kServiceAlertString, userName, action);
    DEBUG.p("cellinfo: " + text);
    if (false) {
      // This takes too long, and doesn't display the alert anyway,
      // and takes people out of apps they're in.
      AlertWindow aw = new AlertWindow(text);
      Registrar.bringToForeground(this);
      aw.setAutoDismiss(1000, null); // WRONG: this will get closed by the returnToSender below...
      aw.show();
    }
    Record record = new Record();
    record.update();
    String signalStrength = record.getSignalStrength();
    String cell = record.getOPER()+","+record.getLAC() + "," + record.getCID();
    ipcMessage.addItem("location", cell + " " + signalStrength);
    ipcMessage.addItem("signalStrength", signalStrength);
    ipcMessage.addItem("cell", cell);
    Registrar.completeIPCReturnToSender(ipcIncoming);
    return handled;
  }

}
